import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import javax.swing.Icon;
import javax.swing.table.DefaultTableCellRenderer;

import java.util.ResourceBundle;
import java.util.MissingResourceException;
import java.util.List;

import CLIPSJNI.*;

public class LightExpert implements ActionListener {

	/* GUI elements */
	private JFrame window;
	private JPanel textPanel;
	private JPanel choicesPanel;
	private JPanel buttonsPanel;
	private JLabel textArea;
	private ButtonGroup choicesGroup;	
	private JButton nextButton;
	//private JButton prevButton;
	private JButton restButton;
	
	/* CLIPS interface */
	private Environment clips;
	
	/* others */
	private ResourceBundle lightsResources;
	private String state = "";
	private String lastState = "";
	private String imageDir = "./images/";
	
	/* konstruktor inicjalizuje i wywietla okineko */
	public LightExpert() {
		// plik z napisami
		try {
			lightsResources = ResourceBundle.getBundle("LightExpert");
        } catch (MissingResourceException mre) {
        	//mre.printStackTrace(); // for debug only
        	JOptionPane.showMessageDialog(null, "Resources file not found.", "Error",
        		    JOptionPane.ERROR_MESSAGE);
        	return;
        }
		
		// okienko
		window = new JFrame("Light Expert");
		window.setBounds(200, 200, 800, 400);
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		
		textPanel = new JPanel();
		textArea = new JLabel(lightsResources.getString("WELCOME"));
		textPanel.add(textArea);
		
		choicesPanel = new JPanel();
		
		// przyciski
		buttonsPanel = new JPanel();
		nextButton = new JButton(lightsResources.getString("START"));
		nextButton.setActionCommand("start");
		nextButton.addActionListener(this);
		restButton = new JButton(lightsResources.getString("RESTART"));
		restButton.setActionCommand("restart");
		restButton.addActionListener(this);
		restButton.setVisible(false);
		buttonsPanel.add(restButton);
		buttonsPanel.add(nextButton);
		
		Container cp = window.getContentPane();
		cp.setLayout(new GridLayout(3, 1));
		cp.add(textPanel);
		cp.add(choicesPanel);
		cp.add(buttonsPanel);
		window.setVisible(true);
		
		// inicjalizacja CLIPS-a
		clips = new Environment();
		clips.load("controler.clp");
		clips.load("lightsBase.clp");
		clips.reset();
		clips.run(1);
		
		nextState();
	}
	
	private void nextState() {
		choicesPanel.removeAll();

		// pobieramy aktualny stan
		MultifieldValue states = (MultifieldValue) clips.eval("(find-all-facts ((?f currentState)) TRUE)");
		@SuppressWarnings("unchecked")
		List<FactAddressValue> statesList = states.listValue();
		if (statesList.isEmpty()) {
			JOptionPane.showMessageDialog(window, "Bd przetwarzania.", "Bd nr 666",
        		    JOptionPane.ERROR_MESSAGE);
		}
		FactAddressValue currentState = statesList.get(0);

		// sprawdzamy czy nie "zablokowalismy" sie na "name"
		state = currentState.getFactSlot("state").toString();
		if (state.contentEquals(lastState)) {
			clips.run(1);
			nextState();
			return;
		}
		lastState = state;

		// tworzenie interfejsu w zaleznosci od stanu
		if (currentState.getFactSlot("state").toString().equals("final")) {
			nextButton.setActionCommand("quit");
			nextButton.setText(lightsResources.getString("QUIT"));
			restButton.setVisible(true);
			choicesPanel.removeAll();

			// wypisywanie wynikow
			MultifieldValue lights = (MultifieldValue) clips.eval("(find-all-facts ((?f result)) TRUE)");
			@SuppressWarnings("unchecked")
			List<FactAddressValue> lightsList = lights.listValue();

			if (!lightsList.isEmpty()) {
				textArea.setText(lightsResources.getString("END"));
				String title = (lightsResources.getString("RESULTS"));
				new ResultsWindow(title, createResultsTable(lightsList));
			} else {
				textArea.setText(lightsResources.getString("NO_RESULTS"));
			}
		} else if (currentState.getFactSlot("state").toString().equals("start")) {
			restButton.setVisible(false);
			nextButton.setActionCommand("start");
			nextButton.setText(lightsResources.getString("START"));
			textArea.setText(lightsResources.getString("WELCOME"));
		}
		else {
			nextButton.setActionCommand("next");
			nextButton.setText(lightsResources.getString("NEXT"));
			restButton.setVisible(true);

			// wstawienie tekstu pytania
			String text = ((SymbolValue) currentState.getFactSlot("text")).stringValue();
			textArea.setText(lightsResources.getString(text));
	
			// wsatawienie kontrolek
			boolean checkboxes = false;
			if (currentState.getFactSlot("choiceType").toString().equals("checkBox")) {
				checkboxes = true;
			}
	
			choicesPanel.removeAll();
			choicesGroup = !checkboxes ? new ButtonGroup() : null;
	
			MultifieldValue choices = (MultifieldValue) currentState.getFactSlot("choices");
			@SuppressWarnings("unchecked")
			List<PrimitiveValue> choicesList = choices.listValue();
			MultifieldValue selectedChoices = (MultifieldValue) currentState.getFactSlot("selected");
			@SuppressWarnings("unchecked")
			List<PrimitiveValue> selectedList = selectedChoices.listValue();
	
			for (PrimitiveValue choice : choicesList) {
				JToggleButton button;
				if (checkboxes) {
					button = new JCheckBox(lightsResources.getString(choice.toString()), false);
				}
				else {
					button = new JRadioButton(lightsResources.getString(choice.toString()), false);
				}
	
				// zaznaczanie domyslnie aktywnych przyciskow	
				for (PrimitiveValue selected : selectedList) {
					if (choice.toString().equals(selected.toString())) {
						button.setSelected(true);
						break;
					}
				}
	
				button.setActionCommand(choice.toString());
				choicesPanel.add(button);
	
				if (!checkboxes) {
					choicesGroup.add(button);
				}
			}
		}
		choicesPanel.repaint();
	}
	
	private JTable createResultsTable(List<FactAddressValue> lightsList) {	
		Object data[][] = new Object[lightsList.size()][2];
		String headers[] = new String[2];
		headers[0] = lightsResources.getString("TABLE_HEADER1");
		headers[1] = lightsResources.getString("TABLE_HEADER2");
		int i = 0;
		for (FactAddressValue light : lightsList) {
			data[i][0] = createImageIcon(imageDir + lightsResources.getString(light.getFactSlot("image").getValue().toString()));
			data[i][1] = lightsResources.getString(light.getFactSlot("description").getValue().toString());
			++i;
		}
		
		JTable resTable = new JTable(data, headers);
		//resTable.getTableHeader().setReorderingAllowed(false);
		resTable.getColumnModel().getColumn(0).setCellRenderer(new IconRenderer());
		resTable.setEnabled(false);
		resTable.setFillsViewportHeight(true);
		return resTable;
	}
	
    private ImageIcon createImageIcon(String path) {
        java.net.URL imgURL = getClass().getResource(path);
        if (imgURL != null) {
            return new ImageIcon(imgURL);
        } else {
            System.err.println("Couldn't find file: " + path);
            return null;
        }
    }

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new LightExpert();
			}
		});
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();
		if (command.equals("next")) {
			// po kliknieciu przycisku "next" dodawane sa fakty do CLIPS-a
			for (Component component : choicesPanel.getComponents()) {
				if (component instanceof JToggleButton) {
					JToggleButton option = (JToggleButton) component;
					if (option.isSelected()) {
						String toAssert = "(" + state + " " + option.getActionCommand() + ")";
						clips.assertString(toAssert);
						System.out.println(toAssert);
					}
				}
			}
			clips.run(1);
			nextState();
		} else if (command.equals("start")) {
			clips.assertString("(start)");
			System.out.println("(start)");
			clips.run(1);
			nextState();
		} else if (command.equals("restart")) {
			clips.reset();
			System.out.println("clips.reset");
			clips.run(1);
			nextState();
		} else if (command.equals("quit")) {
			window.dispose();
		}
	}
}

class IconRenderer extends DefaultTableCellRenderer {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1352134277004197571L;

	public IconRenderer() {
		super();
	}
	
	public void setValue(Object value) {
		if (value == null) {
			setText("");
		} else {
			setIcon((Icon) value);
		}
	}
}

class ResultsWindow extends JFrame {
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -3444185031942634323L;
	private JScrollPane scrollPane;
	
	public ResultsWindow(String title, JTable resTable) {
		super(title);
		setBounds(0, 0, 1000, 800);
		//setExtendedState(this.getExtendedState() | Frame.MAXIMIZED_BOTH);
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		Container cp = getContentPane();
		resTable.setRowHeight(300);
		resTable.getColumnModel().getColumn(0).setPreferredWidth(300);
		resTable.getColumnModel().getColumn(1).setPreferredWidth(600);
		scrollPane = new JScrollPane(resTable);
		cp.add(scrollPane);
		setVisible(true);
	}
}