(deftemplate currentState
	(slot name (default none))
	(slot state (default question))
	(slot text)		
	(slot answerBoxType (default radioButton))
	(multislot selectedAnswers)
	(multislot answers)
)


(deftemplate result
	(slot brand (type STRING))
	(slot code (type STRING))
)


; Startup window
(defrule system-banner ""
	=>
	(assert	(currentState 
		(text START_TEXT)
		(name start)
		(state init)
		(answers))
	)
)


; Results window
(defrule EndOfQuestions ""
	?state<-(currentState)
	(end)
	=>
	(retract ?state) 
	(assert (currentState 
		(text NO_RESULTS_END_TEXT)
		(state final)
	))
)


; Type question
(defrule selectType ""
	?state<-(currentState)
	(not(typeQuestion))
	=>
	(retract ?state) 
	(assert (typeQuestion))	
	(assert (currentState
		(text SELECT_TYPE_QUESTION)
		(name type)
		(answers NORMAL SPECIAL)
		(selectedAnswers NORMAL)	
	))
)

; Gender question
(defrule selectGender ""
	?state<-(currentState)
	(typeQuestion)
	(type NORMAL)
	(not(genderQuestion))
	=>
	(retract ?state) 
	(assert (genderQuestion))	
	(assert (currentState
		(name gender)	
		(text SELECT_GENDER_QUESTION)
		(answers MALE FEMALE KIDS)
		(selectedAnswers MALE)	
	))
)

; Price question
(defrule selectPrice ""
	?state<-(currentState)
	(genderQuestion)
	(type NORMAL)
	(not(priceQuestion))
	=>
	(retract ?state) 
	(assert (priceQuestion))	 c
	(assert (currentState
		(text SELECT_PRICE_QUESTION)
		(name price)
		(answers CHEAP MEDIUM EXPENSIVE EXCLUSIVE ANY)
		(selectedAnswers ANY)	
	))
)

; Style question
(defrule selectStyle ""
	?state<-(currentState)
	(priceQuestion)
	(not(styleQuestion))
	=>
	(retract ?state) 
	(assert (styleQuestion))	
	(assert (currentState
		(text SELECT_STYLE_QUESTION)
		(name style)
		(answers SPORT SMART_AND_SPORT ELEGANT FASHION ANY)
		(selectedAnswers ANY)	
	))
)

; Mechanism question
(defrule selectMechanism ""
	?state<-(currentState)
	(styleQuestion)
	(not(mechanismQuestion))
	=>
	(retract ?state) 
	(assert (mechanismQuestion))	
	(assert (currentState
		(text SELECT_MECHANISM_QUESTION)
		(answerBoxType checkBox)
		(name mechanism)
		(answers QUARTZ SOLAR_QUARTZ AUTOMATIC QUARTZ_KINETIC MECHANICAL)
		(selectedAnswers QUARTZ SOLAR_QUARTZ AUTOMATIC QUARTZ_KINETIC MECHANICAL)	
	))
)

; ShieldShape question
(defrule selectShieldShape ""
	?state<-(currentState)
	(mechanismQuestion)
	(not(shapeQuestion))
	=>
	(retract ?state) 
	(assert (shapeQuestion))	
	(assert (currentState
		(text SELECT_SHAPE_QUESTION)
		(name shieldShape )
		(answers OVAL SQUARE ANY)
		(selectedAnswers ANY)	
	))
)

; ShieldSize question
(defrule selectSize ""
	?state<-(currentState)
	(shapeQuestion)
	(not(sizeQuestion))
	=>
	(retract ?state) 
	(assert (sizeQuestion))	
	(assert (currentState
		(text SELECT_SIZE_QUESTION)
		(name shieldDimension)
		(answers SMALL MEDIUM LARGE HUGE ANY)
		(selectedAnswers ANY)	
	))
)


; ShieldMaterial question
(defrule selectShieldMaterial ""
	?state<-(currentState)
	(sizeQuestion)
	(not(shieldMaterialQuestion))
	=>
	(retract ?state) 
	(assert (shieldMaterialQuestion))	
	(assert (currentState
		(text SELECT_SHMATERIAL_QUESTION)
		(answerBoxType checkBox)
		(name shieldMaterial)
		(answers STEEL TITAN PLASTIC CARBON STEEL_AND_PLASTIC STEEL_AND_TITAN STEEL_AND_CERAMICS STEEL_AND_PVD CERAMICS BRASS STEEL_AND_GOLD ALUMINIUM)
		(selectedAnswers STEEL TITAN PLASTIC CARBON STEEL_AND_PLASTIC STEEL_AND_TITAN STEEL_AND_CERAMICS STEEL_AND_PVD CERAMICS BRASS STEEL_AND_GOLD ALUMINIUM)	
	))
)

; StrapMaterial question
(defrule selectStrapMaterial ""
	?state<-(currentState)
	(shieldMaterialQuestion)
	(type NORMAL)
	(not(strapMaterialQuestion))
	=>
	(retract ?state) 
	(assert (strapMaterialQuestion))	
	(assert (currentState
		(text SELECT_STMATERIAL_QUESTION)
		(answerBoxType checkBox)
		(name strapMaterial)
		(answers LEATHER STEEL TITAN PLASTIC LEATHER_AND_PLASTIC LEATHER_AND_MATERIAL STEEL_AND_LEATHER STEEL_AND_PLASTIC STEEL_AND_CERAMICS STEEL_AND_PVD CERAMICS MATERIAL RUBBER)
		(selectedAnswers LEATHER STEEL TITAN PLASTIC LEATHER_AND_PLASTIC LEATHER_AND_MATERIAL STEEL_AND_LEATHER STEEL_AND_PLASTIC STEEL_AND_CERAMICS STEEL_AND_PVD CERAMICS MATERIAL RUBBER)	
	))
)

; Glass question
(defrule selectGlass ""
	?state<-(currentState)
	(strapMaterialQuestion)
	(not(glassQuestion))
	=>
	(retract ?state) 
	(assert (glassQuestion))	
	(assert (currentState
		(text SELECT_GLASS_QUESTION)
		(answerBoxType checkBox)
		(name glass)
		(answers MINERAL SAPPHIRE PLASTIC HEXANE PLEXI)
		(selectedAnswers MINERAL SAPPHIRE PLASTIC HEXANE PLEXI)	
	))
)

; Waterproof question
(defrule selectWaterproof ""
	?state<-(currentState)
	(not(waterproofQuestion))
	(glassQuestion)
	=>
	(retract ?state) 
	(assert (waterproofQuestion))	
	(assert (currentState
		(name waterproof)	
		(text SELECT_WATERPROOF_QUESTION)
		(answers LOW MEDIUM HIGH SPLASHPROOF ANY)
		(selectedAnswers ANY)	
	))
)


; Func question
(defrule selectFunc ""
	?state<-(currentState)
	(waterproofQuestion)
	(type NORMAL)
	(not(funcQuestion))
	=>
	(retract ?state) 
	(assert (funcQuestion))	
	(assert (currentState
		(text SELECT_FUNC_QUESTION)
		(name func)
		(answers NO YES)
		(selectedAnswers NO)	
	))
)
; FuncNO question
(defrule selectFuncNO ""
	(func NO)
	(not(funNO))
	=>
	(assert(waterproof ANY))
	(assert(funNO))
	(assert (end))
)

; Functions question
(defrule selectFunctions ""
	?state<-(currentState)
	(not(functionsQuestion))
	(funcQuestion)
	(func YES)
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM TIMER STOPWATCH DATE_STAMP)
		(selectedAnswers BACKLIGHT ALARM TIMER STOPWATCH DATE_STAMP)	
	))
	(assert (end))
)

; GenderSpec question
(defrule selectGenderSpec ""
	?state<-(currentState)
	(typeQuestion)
	(type SPECIAL)
	(not(genderQuestion))
	=>
	(retract ?state) 
	(assert (genderQuestion))	
	(assert (currentState
		(name gender)	
		(text SELECT_GENDER_QUESTION)
		(answers MALE FEMALE)
		(selectedAnswers MALE)	
	))
)

; Activity question
(defrule selectActivity ""
	?state<-(currentState)
	(type SPECIAL)
	(genderQuestion)
	(not(activityQuestion))
	=>
	(retract ?state) 
	(assert (activityQuestion))	
	(assert (currentState
		(name activity)	
		(text SELECT_ACTIVITY_QUESTION)
		(answers SAILING FISHING TRAINING DIVING SITE_SEEING TREKKING)
		(selectedAnswers TRAINING)	
	))
)

; ShieldMaterialSpec question
(defrule selectShieldMaterialSpec ""
	?state<-(currentState)
	(activityQuestion)
	(not(shieldMaterialQuestion))
	=>
	(retract ?state) 
	(assert (shieldMaterialQuestion))	
	(assert (currentState
		(text SELECT_SHMATERIAL_QUESTION)
		(answerBoxType checkBox)
		(name shieldMaterial)
		(answers STEEL TITAN PLASTIC)
		(selectedAnswers STEEL TITAN PLASTIC)	
	))
)

; StrapMaterialSpec question
(defrule selectStrapMaterialSpec""
	?state<-(currentState)
	(shieldMaterialQuestion)
	(type SPECIAL)
	(not(strapMaterialQuestion))
	=>
	(retract ?state) 
	(assert (strapMaterialQuestion))	
	(assert (currentState
		(text SELECT_STMATERIAL_QUESTION)
		(answerBoxType checkBox)
		(name strapMaterial)
		(answers LEATHER STEEL PLASTIC MATERIAL RUBBER)
		(selectedAnswers LEATHER STEEL PLASTIC MATERIAL RUBBER)	
	))
)

; FunctionsSail question
(defrule selectFunctionsSail ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity SAILING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH DATE_STAMP BAROMETER COMPASS REGATTA RATE_OF_SEA_FISHING)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH DATE_STAMP BAROMETER COMPASS REGATTA RATE_OF_SEA_FISHING)	
	))
	(assert (end))
)

; FunctionsFish question
(defrule selectFunctionsFish ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity FISHING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER FISHING_TIME MOON_PHASES THERMOMETER)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER FISHING_TIME MOON_PHASES THERMOMETER)	
	))
	(assert (end))
)

; FunctionsTrain question
(defrule selectFunctionsTrain ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity TRAINING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH TIMER LAP_MEMORY CALORIES_MEASURE HEART_MONITOR)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH TIMER LAP_MEMORY CALORIES_MEASURE HEART_MONITOR)	
	))
	(assert (end))
)

; FunctionsDive question
(defrule selectFunctionsDive ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity DIVING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH DATE_STAMP TACHYMETER COMPASS DEPTH_MEASURE LOG_DIVES THERMOMETER)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH DATE_STAMP TACHYMETER COMPASS DEPTH_MEASURE LOG_DIVES THERMOMETER)	
	))
	(assert (end))
)

; FunctionsSiteSee question
(defrule selectFunctionsSiteSee ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity SITE_SEEING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER BAROMETER COMPASS THERMOMETER ALTIMETER TIDES)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER BAROMETER COMPASS THERMOMETER ALTIMETER TIDES)	
	))
	(assert (end))
)

; FunctionsTrek question
(defrule selectFunctionsTrek ""
	?state<-(currentState)
	(waterproofQuestion)
	(activity TREKKING)
	(not(functionsQuestion))
	=>
	(retract ?state) 
	(assert (functionsQuestion))	
	(assert (currentState
		(name functions)	
		(text SELECT_FUNCTIONS_QUESTION)
		(answerBoxType checkBox)
		(answers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER BAROMETER ALTIMETER DEPTH_MEASURE COMPASS THERMOMETER HEART_MONITOR)
		(selectedAnswers BACKLIGHT ALARM STOPWATCH DATE_STAMP TIMER BAROMETER ALTIMETER DEPTH_MEASURE COMPASS THERMOMETER HEART_MONITOR)	
	))
	(assert (end))
)