import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import CLIPSJNI.Environment;
import CLIPSJNI.FactAddressValue;
import CLIPSJNI.MultifieldValue;
import CLIPSJNI.PrimitiveValue;


public class BaseState
{
	private Environment clips;
	private Properties properties;
	FactAddressValue currentBaseState;
	
	
	
	public BaseState(Environment clips, Properties properties) {
		super();
		this.clips = clips;
		this.properties = properties;
	}
	
	public boolean update()
	{
		MultifieldValue currentStateMV = (MultifieldValue) clips
				.eval("(find-all-facts ((?f currentState)) TRUE)");

		if (currentStateMV.listValue().isEmpty())
			return false;
		
		currentBaseState = (FactAddressValue) currentStateMV.listValue().get(0);
		return true;
	}
	
	public String getState()
	{
		return currentBaseState.getFactSlot("state").toString();
	}
	
	public String getText()
	{
		String text = currentBaseState.getFactSlot("text").toString();
		return properties.getProperty(text); 
	}
	
	public String getAnswerBoxType()
	{
		return currentBaseState.getFactSlot("answerBoxType").toString();
	}
	
	public List<String> getAnswersList()
	{
		MultifieldValue answersListMV = (MultifieldValue) currentBaseState
				.getFactSlot("answers");
		List<PrimitiveValue> answersList = answersListMV.listValue();
		List<String> answersStringList = new ArrayList<String>();
		
		for (PrimitiveValue answer : answersList)
			answersStringList.add(answer.toString());
		
		return answersStringList;
	}
	
	public List<String> getSelectedAnswersList()
	{
		MultifieldValue selectedMV = (MultifieldValue) currentBaseState
				.getFactSlot("selectedAnswers");
		List<PrimitiveValue> selectedList = selectedMV.listValue();
		List<String> selectedStringList = new ArrayList<String>();
		
		for (PrimitiveValue selected : selectedList)
			selectedStringList.add(selected.toString());
		
		return selectedStringList;
	}
	
	public String getName()
	{
		return currentBaseState.getFactSlot("name").toString();
	}
	
	public boolean equals(Object obj)
	{
        if (obj == null || getClass() != obj.getClass())
            return false;
 
        if (getName().equals(((BaseState) obj).getName()))
            return true;
        
        return false;
    }
	
	public Object clone() {
		BaseState newBaseState;
		newBaseState = new BaseState(clips, properties);
		newBaseState.currentBaseState = currentBaseState;
		
	    return newBaseState;
	  }
}
