import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Properties;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.SwingUtilities;
import javax.swing.table.TableColumn;

import CLIPSJNI.Environment;
import CLIPSJNI.FactAddressValue;
import CLIPSJNI.MultifieldValue;

public class WristWatches implements ActionListener {
	private JFrame mainFrame;

	private JPanel textPanel;
	private JPanel answerPanel;
	private JPanel actionPanel;

	private JTextPane textContainer;
	private ButtonGroup answerButtonGroup;
	private JButton actionButton;

	private Environment clips;
	private Properties properties;

	BaseState currentBaseState;
	BaseState lastCurrentBaseState;

	public WristWatches() {
		initMainFrames();
		initClips();

		// odczytanie stanu bazy
		nextState();
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new WristWatches();
			}
		});
	}

	private void nextState() {
		answerPanel.removeAll();

		// zaktualizowanie obecnego stanu bazy
		if (!currentBaseState.update())
			return;

		System.out.println("current: " + currentBaseState.getName());
		if (lastCurrentBaseState != null)
			System.out.println("last: " + lastCurrentBaseState.getName());

		while (currentBaseState.equals(lastCurrentBaseState)) {
			System.out.println("Oczekuje na nowy state");
			clips.run(1);
			currentBaseState.update();
		}
		lastCurrentBaseState = (BaseState) currentBaseState.clone();

		System.out.println("Stan systemu: \n\tname: "
				+ currentBaseState.getName() + "\n\tstate: "
				+ currentBaseState.getState() + "\n");

		// ustawienie odpowiedniego stanu dla przycisku
		if (currentBaseState.getState().equals("init")) {
			actionButton.setText(properties.getProperty("BUTTON_TEXT_START")
					.toString());
			actionButton.setActionCommand("start");
		}
		if (currentBaseState.getState().equals("question")) {
			actionButton
					.setText(properties.getProperty("BUTTON_TEXT_QUESTION"));
			actionButton.setActionCommand("next");
		}
		if (currentBaseState.getState().equals("final")) {
			actionButton.setText(properties.getProperty("BUTTON_TEXT_RESTART"));
			actionButton.setActionCommand("restart");
			printResults();

			answerPanel.repaint();
			return;
		}

		// ustawienie tekstu dla textContainer
		textContainer.setText(currentBaseState.getText());

		// pobranie listy odpowiedzi
		List<String> answersList = currentBaseState.getAnswersList();

		// pobranie listy zaznaczonych
		List<String> selectedAnswersList = currentBaseState
				.getSelectedAnswersList();

		answerButtonGroup = new ButtonGroup();

		for (String answer : answersList) {
			JToggleButton box;
			if (currentBaseState.getAnswerBoxType().equals("checkBox")) {
				box = new JCheckBox(properties.getProperty(answer),
						checkSelection(selectedAnswersList, answer));
			} else {
				box = new JRadioButton(properties.getProperty(answer),
						checkSelection(selectedAnswersList, answer));
				answerButtonGroup.add(box);
			}

			box.setActionCommand(answer);
			answerPanel.add(box);
		}

		answerPanel.repaint();
	}

	private void printResults() {
		// dodatkowo wypisywanie wynikow
		MultifieldValue watchesMV = (MultifieldValue) clips
				.eval("(find-all-facts ((?f result)) TRUE)");
		List<FactAddressValue> watchesList = watchesMV.listValue();

		if (watchesList.isEmpty()) {
			textContainer
					.setText(properties.getProperty("NO_RESULTS_END_TEXT"));
		} else {
			textContainer.setText(properties.getProperty("RESULTS_END_TEXT"));

			int colCount = Integer.parseInt(properties
					.getProperty("RESULT_TABLE_COL_COUNT"));

			String resultHeader[] = new String[colCount];
			String resultData[][] = new String[watchesList.size()][colCount];

			// pobranie nazw naglowkow tabeli
			for (int i = 0; i < colCount; i++) {
				resultHeader[i] = properties.getProperty("RESULT_TABLE_COL"
						+ (i + 1) + "_HEADER");
			}

			// pobranie danych
			for (int i = 0; i < watchesList.size(); i++) {
				for (int j = 0; j < colCount; j++) {
					String factSlotName = properties
							.getProperty("RESULT_TABLE_COL" + (j + 1)
									+ "_FACT_SLOT_NAME");
					resultData[i][j] = watchesList.get(i)
							.getFactSlot(factSlotName).getValue().toString();
				}
			}

			JTable resultTable = new JTable(resultData, resultHeader);
			resultTable.setPreferredScrollableViewportSize(new Dimension(
					answerPanel.getWidth() - 25, answerPanel.getHeight() - 25));
			resultTable.setEnabled(false);
			resultTable.setAutoCreateRowSorter(true);

			TableColumn column = new TableColumn();
			column = resultTable.getColumnModel().getColumn(0);
			column.setMaxWidth(100);

			JScrollPane scrollPane = new JScrollPane(resultTable);
			answerPanel.add(scrollPane);
		}
	}

	private boolean checkSelection(List<String> selectedList, String answer) {
		for (String selected : selectedList) {
			if (selected.equals(answer)) {
				return true;
			}
		}

		return false;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getActionCommand().equals("start")) {
			clips.assertString("(start)");
			System.out.println("(start)");
		}
		if (e.getActionCommand().equals("next")) {
			boolean wasToggleButton = false;
			List<String> toAssertList = new ArrayList<>();
			for (Component component : answerPanel.getComponents()) {
				if (component instanceof JToggleButton) {
					wasToggleButton = true;
					JToggleButton box = (JToggleButton) component;
					if (box.isSelected()) {
						String stateName = currentBaseState.getName();
						toAssertList.add("(" + stateName + " "
								+ box.getActionCommand() + ")");
					}
				}
			}
			if (toAssertList.isEmpty() && wasToggleButton) {
				JOptionPane.showConfirmDialog(null,
						"Please select at least one field!", "Selection error",
						JOptionPane.CLOSED_OPTION, JOptionPane.ERROR_MESSAGE);
				return;
			} else {
				for (String toAssert : toAssertList) {
					clips.assertString(toAssert);
					System.out.println(toAssert);
				}
			}
		}
		if (e.getActionCommand().equals("restart")) {
			clips.reset();
			System.out.println("clips.reset");
		}

		clips.run(1);
		nextState();
	}

	private void initMainFrames() {
		// okno gwne
		mainFrame = new JFrame("Wrist Watches Expert System");
		mainFrame.getContentPane().setLayout(new GridLayout(3, 1));
		mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		mainFrame.setSize(400, 450);
		mainFrame.setLocationRelativeTo(null);
		mainFrame.setResizable(false);

		// panel do wpisywania pyta
		textPanel = new JPanel();
		textContainer = new JTextPane();
		textContainer.setMargin(new Insets(30, 10, 10, 10));
		textContainer.setOpaque(false);
		textContainer.setAutoscrolls(false);
		textContainer.setEditable(false);

		textPanel.add(textContainer);
		mainFrame.add(textPanel);

		// panel do odpowiedzi
		answerPanel = new JPanel();
		answerButtonGroup = new ButtonGroup();

		mainFrame.add(answerPanel);
		
		// panel do akcji
		actionPanel = new JPanel();
		actionPanel.setSize(400, 100);
		actionButton = new JButton();
		actionButton.addActionListener(this);
		
		actionPanel.setLayout(new GridBagLayout());
        GridBagConstraints gbc = new GridBagConstraints();

		actionPanel.add(actionButton, gbc);
		mainFrame.add(actionPanel);

		// okno gwne inicjalizacja
		mainFrame.setVisible(true);
	}

	private void initClips() {
		clips = new Environment();
		clips.load("control.clp");
		clips.load("knowledgeBase.clp");
		clips.reset();
		clips.run(1);

		properties = new Properties();
		loadProperties(new File("properties"));

		lastCurrentBaseState = null;
		currentBaseState = new BaseState(clips, properties);
	}

	private void loadProperties(File propertiesFile) {
		InputStream is;
		try {
			is = new FileInputStream(propertiesFile);
			properties.load(is);
		} catch (FileNotFoundException e) {
			System.out.println("Cannot find configuration file.");
		} catch (IOException e) {
			System.out.println("Some problems with configuration file.");
		}
	}
}
